import { HttpClient } from '@angular/common/http';
import { ApolloLink, Observable } from 'apollo-link';
import { BatchLink } from 'apollo-link-batch';
import { print } from 'graphql/language/printer';
import { fetch, mergeHeaders, prioritize } from 'apollo-angular-link-http-common';
import { Injectable, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const defaults = {
    batchInterval: 10,
    batchMax: 10,
    uri: 'graphql',
    method: 'POST',
};
class HttpBatchLinkHandler extends ApolloLink {
    /**
     * @param {?} httpClient
     * @param {?} options
     */
    constructor(httpClient, options) {
        super();
        this.httpClient = httpClient;
        this.options = options;
        this.batchInterval = options.batchInterval || defaults.batchInterval;
        this.batchMax = options.batchMax || defaults.batchMax;
        /** @type {?} */
        const batchHandler = (/**
         * @param {?} operations
         * @return {?}
         */
        (operations) => {
            return new Observable((/**
             * @param {?} observer
             * @return {?}
             */
            (observer) => {
                /** @type {?} */
                const body = this.createBody(operations);
                /** @type {?} */
                const headers = this.createHeaders(operations);
                const { method, uri, withCredentials } = this.createOptions(operations);
                if (typeof uri === 'function') {
                    throw new Error(`Option 'uri' is a function, should be a string`);
                }
                /** @type {?} */
                const req = {
                    method,
                    url: uri,
                    body: body,
                    options: {
                        withCredentials,
                        headers,
                    },
                };
                /** @type {?} */
                const sub = fetch(req, this.httpClient).subscribe({
                    next: (/**
                     * @param {?} result
                     * @return {?}
                     */
                    result => observer.next(result.body)),
                    error: (/**
                     * @param {?} err
                     * @return {?}
                     */
                    err => observer.error(err)),
                    complete: (/**
                     * @return {?}
                     */
                    () => observer.complete()),
                });
                return (/**
                 * @return {?}
                 */
                () => {
                    if (!sub.closed) {
                        sub.unsubscribe();
                    }
                });
            }));
        });
        /** @type {?} */
        const batchKey = options.batchKey ||
            ((/**
             * @param {?} operation
             * @return {?}
             */
            (operation) => {
                return this.createBatchKey(operation);
            }));
        this.batcher = new BatchLink({
            batchInterval: this.batchInterval,
            batchMax: this.batchMax,
            batchKey,
            batchHandler,
        });
    }
    /**
     * @private
     * @param {?} operations
     * @return {?}
     */
    createOptions(operations) {
        /** @type {?} */
        const context = operations[0].getContext();
        return {
            method: prioritize(context.method, this.options.method, defaults.method),
            uri: prioritize(context.uri, this.options.uri, defaults.uri),
            withCredentials: prioritize(context.withCredentials, this.options.withCredentials),
        };
    }
    /**
     * @private
     * @param {?} operations
     * @return {?}
     */
    createBody(operations) {
        return operations.map((/**
         * @param {?} operation
         * @return {?}
         */
        operation => {
            /** @type {?} */
            const includeExtensions = prioritize(operation.getContext().includeExtensions, this.options.includeExtensions, false);
            /** @type {?} */
            const includeQuery = prioritize(operation.getContext().includeQuery, this.options.includeQuery, true);
            /** @type {?} */
            const body = {
                operationName: operation.operationName,
                variables: operation.variables,
            };
            if (includeExtensions) {
                body.extensions = operation.extensions;
            }
            if (includeQuery) {
                body.query = print(operation.query);
            }
            return body;
        }));
    }
    /**
     * @private
     * @param {?} operations
     * @return {?}
     */
    createHeaders(operations) {
        return operations.reduce((/**
         * @param {?} headers
         * @param {?} operation
         * @return {?}
         */
        (headers, operation) => {
            return mergeHeaders(headers, operation.getContext().headers);
        }), this.options.headers);
    }
    /**
     * @private
     * @param {?} operation
     * @return {?}
     */
    createBatchKey(operation) {
        /** @type {?} */
        const context = operation.getContext();
        if (context.skipBatching) {
            return Math.random()
                .toString(36)
                .substr(2, 9);
        }
        /** @type {?} */
        const headers = context.headers &&
            context.headers.keys().map((/**
             * @param {?} k
             * @return {?}
             */
            (k) => context.headers.get(k)));
        /** @type {?} */
        const opts = JSON.stringify({
            includeQuery: context.includeQuery,
            includeExtensions: context.includeExtensions,
            headers,
        });
        return prioritize(context.uri, this.options.uri) + opts;
    }
    /**
     * @param {?} op
     * @return {?}
     */
    request(op) {
        return this.batcher.request(op);
    }
}
class HttpBatchLink {
    /**
     * @param {?} httpClient
     */
    constructor(httpClient) {
        this.httpClient = httpClient;
    }
    /**
     * @param {?} options
     * @return {?}
     */
    create(options) {
        return new HttpBatchLinkHandler(this.httpClient, options);
    }
}
HttpBatchLink.decorators = [
    { type: Injectable }
];
/** @nocollapse */
HttpBatchLink.ctorParameters = () => [
    { type: HttpClient }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const PROVIDERS = [HttpBatchLink];
class HttpBatchLinkModule {
}
HttpBatchLinkModule.decorators = [
    { type: NgModule, args: [{
                providers: PROVIDERS,
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { HttpBatchLink, HttpBatchLinkHandler, HttpBatchLinkModule, PROVIDERS };

//# sourceMappingURL=ng.apolloLink.httpBatch.js.map