import { __extends } from 'tslib';
import { HttpClient } from '@angular/common/http';
import { ApolloLink, Observable } from 'apollo-link';
import { BatchLink } from 'apollo-link-batch';
import { print } from 'graphql/language/printer';
import { fetch, mergeHeaders, prioritize } from 'apollo-angular-link-http-common';
import { Injectable, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var defaults = {
    batchInterval: 10,
    batchMax: 10,
    uri: 'graphql',
    method: 'POST',
};
var HttpBatchLinkHandler = /** @class */ (function (_super) {
    __extends(HttpBatchLinkHandler, _super);
    function HttpBatchLinkHandler(httpClient, options) {
        var _this = _super.call(this) || this;
        _this.httpClient = httpClient;
        _this.options = options;
        _this.batchInterval = options.batchInterval || defaults.batchInterval;
        _this.batchMax = options.batchMax || defaults.batchMax;
        /** @type {?} */
        var batchHandler = (/**
         * @param {?} operations
         * @return {?}
         */
        function (operations) {
            return new Observable((/**
             * @param {?} observer
             * @return {?}
             */
            function (observer) {
                /** @type {?} */
                var body = _this.createBody(operations);
                /** @type {?} */
                var headers = _this.createHeaders(operations);
                var _a = _this.createOptions(operations), method = _a.method, uri = _a.uri, withCredentials = _a.withCredentials;
                if (typeof uri === 'function') {
                    throw new Error("Option 'uri' is a function, should be a string");
                }
                /** @type {?} */
                var req = {
                    method: method,
                    url: uri,
                    body: body,
                    options: {
                        withCredentials: withCredentials,
                        headers: headers,
                    },
                };
                /** @type {?} */
                var sub = fetch(req, _this.httpClient).subscribe({
                    next: (/**
                     * @param {?} result
                     * @return {?}
                     */
                    function (result) { return observer.next(result.body); }),
                    error: (/**
                     * @param {?} err
                     * @return {?}
                     */
                    function (err) { return observer.error(err); }),
                    complete: (/**
                     * @return {?}
                     */
                    function () { return observer.complete(); }),
                });
                return (/**
                 * @return {?}
                 */
                function () {
                    if (!sub.closed) {
                        sub.unsubscribe();
                    }
                });
            }));
        });
        /** @type {?} */
        var batchKey = options.batchKey ||
            ((/**
             * @param {?} operation
             * @return {?}
             */
            function (operation) {
                return _this.createBatchKey(operation);
            }));
        _this.batcher = new BatchLink({
            batchInterval: _this.batchInterval,
            batchMax: _this.batchMax,
            batchKey: batchKey,
            batchHandler: batchHandler,
        });
        return _this;
    }
    /**
     * @private
     * @param {?} operations
     * @return {?}
     */
    HttpBatchLinkHandler.prototype.createOptions = /**
     * @private
     * @param {?} operations
     * @return {?}
     */
    function (operations) {
        /** @type {?} */
        var context = operations[0].getContext();
        return {
            method: prioritize(context.method, this.options.method, defaults.method),
            uri: prioritize(context.uri, this.options.uri, defaults.uri),
            withCredentials: prioritize(context.withCredentials, this.options.withCredentials),
        };
    };
    /**
     * @private
     * @param {?} operations
     * @return {?}
     */
    HttpBatchLinkHandler.prototype.createBody = /**
     * @private
     * @param {?} operations
     * @return {?}
     */
    function (operations) {
        var _this = this;
        return operations.map((/**
         * @param {?} operation
         * @return {?}
         */
        function (operation) {
            /** @type {?} */
            var includeExtensions = prioritize(operation.getContext().includeExtensions, _this.options.includeExtensions, false);
            /** @type {?} */
            var includeQuery = prioritize(operation.getContext().includeQuery, _this.options.includeQuery, true);
            /** @type {?} */
            var body = {
                operationName: operation.operationName,
                variables: operation.variables,
            };
            if (includeExtensions) {
                body.extensions = operation.extensions;
            }
            if (includeQuery) {
                body.query = print(operation.query);
            }
            return body;
        }));
    };
    /**
     * @private
     * @param {?} operations
     * @return {?}
     */
    HttpBatchLinkHandler.prototype.createHeaders = /**
     * @private
     * @param {?} operations
     * @return {?}
     */
    function (operations) {
        return operations.reduce((/**
         * @param {?} headers
         * @param {?} operation
         * @return {?}
         */
        function (headers, operation) {
            return mergeHeaders(headers, operation.getContext().headers);
        }), this.options.headers);
    };
    /**
     * @private
     * @param {?} operation
     * @return {?}
     */
    HttpBatchLinkHandler.prototype.createBatchKey = /**
     * @private
     * @param {?} operation
     * @return {?}
     */
    function (operation) {
        /** @type {?} */
        var context = operation.getContext();
        if (context.skipBatching) {
            return Math.random()
                .toString(36)
                .substr(2, 9);
        }
        /** @type {?} */
        var headers = context.headers &&
            context.headers.keys().map((/**
             * @param {?} k
             * @return {?}
             */
            function (k) { return context.headers.get(k); }));
        /** @type {?} */
        var opts = JSON.stringify({
            includeQuery: context.includeQuery,
            includeExtensions: context.includeExtensions,
            headers: headers,
        });
        return prioritize(context.uri, this.options.uri) + opts;
    };
    /**
     * @param {?} op
     * @return {?}
     */
    HttpBatchLinkHandler.prototype.request = /**
     * @param {?} op
     * @return {?}
     */
    function (op) {
        return this.batcher.request(op);
    };
    return HttpBatchLinkHandler;
}(ApolloLink));
var HttpBatchLink = /** @class */ (function () {
    function HttpBatchLink(httpClient) {
        this.httpClient = httpClient;
    }
    /**
     * @param {?} options
     * @return {?}
     */
    HttpBatchLink.prototype.create = /**
     * @param {?} options
     * @return {?}
     */
    function (options) {
        return new HttpBatchLinkHandler(this.httpClient, options);
    };
    HttpBatchLink.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    HttpBatchLink.ctorParameters = function () { return [
        { type: HttpClient }
    ]; };
    return HttpBatchLink;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var PROVIDERS = [HttpBatchLink];
var HttpBatchLinkModule = /** @class */ (function () {
    function HttpBatchLinkModule() {
    }
    HttpBatchLinkModule.decorators = [
        { type: NgModule, args: [{
                    providers: PROVIDERS,
                },] }
    ];
    return HttpBatchLinkModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { HttpBatchLink, HttpBatchLinkHandler, HttpBatchLinkModule, PROVIDERS };

//# sourceMappingURL=ng.apolloLink.httpBatch.js.map